Package: hmmTMB
Type: Package
Title: Fit Hidden Markov Models using Template Model Builder
Version: 1.1.2
Authors@R: c(
  person(given = "Theo",
         family = "Michelot",
         role = c("aut", "cre"),
         email = "theo.michelot@dal.ca"),
  person(given = "Richard",
         family = "Glennie",
         role = c("aut", "ctb")))
Maintainer: Theo Michelot <theo.michelot@dal.ca>
Description: Fitting hidden Markov models using automatic differentiation 
  and Laplace approximation, allowing for fast inference and flexible covariate
  effects (including random effects and smoothing splines) on model parameters.
  The package is described by Michelot (2025) <doi:10.18637/jss.v114.i05>.
URL: https://github.com/TheoMichelot/hmmTMB
License: GPL-3
Depends: R6, mgcv, TMB, ggplot2
Imports: Matrix, stringr, MASS, tmbstan, methods
LinkingTo: TMB, RcppEigen
Suggests: rstan, testthat, knitr, rmarkdown, moveHMM, scico, MSwM,
        unmarked
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-11-04 14:27:49 UTC; theo
Author: Theo Michelot [aut, cre],
  Richard Glennie [aut, ctb]
Repository: CRAN
Date/Publication: 2025-11-05 13:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2025-11-05 17:09:30 UTC; unix
Archs: hmmTMB.so.dSYM
