# Generated by roxygen2: do not edit by hand

S3method(logLik,HMM)
S3method(update,HMM)
export(Dist)
export(HMM)
export(MarkovChain)
export(Observation)
export(as_character_formula)
export(invmlogit)
export(is_whole_number)
export(logsumexp)
export(make_formulas)
export(mlogit)
export(mvnorm_invlink)
export(mvnorm_link)
export(prec_to_cov)
export(quad_pos_solve)
export(strip_comments)
importFrom(MASS,ginv)
importFrom(Matrix,bdiag)
importFrom(R6,R6Class)
importFrom(TMB,MakeADFun)
importFrom(TMB,sdreport)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,label_bquote)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(methods,as)
importFrom(mgcv,dmvn)
importFrom(mgcv,gam)
importFrom(mgcv,ldTweedie)
importFrom(mgcv,rTweedie)
importFrom(mgcv,rmvn)
importFrom(stats,as.formula)
importFrom(stats,nlminb)
importFrom(stats,plogis)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stringr,str_split)
importFrom(stringr,str_split_fixed)
importFrom(stringr,str_trim)
importFrom(tmbstan,tmbstan)
useDynLib(hmmTMB, .registration = TRUE)
